#ifdef __APPLE__
#include <GLUT/glut.h> // Apple's version of GLUT is here
#else
#include <GL/glut.h> // but the standard place for it is here
#endif

// On some platforms like Windows, this never gets defined in a standard place
#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#include "marshmallow_man.h"

//============================================================================
// global variables, used by UI functions

int winwidth=640, winheight=480; // the pixel dimensions of the window
float fovy=45; // field of view in y
float near_clip=1, far_clip=20; // z range of camera
float heading=0; // camera heading (rotation around world's y-axis)
float pitch=0; // pitch (rotation around camera's x-axis)
float camera_dist=10; // how far the camera is from the world-space origin
int old_mouse_x=0; // last known mouse cursor position in x
int old_mouse_y=0; // last known mouse cursor position in y

//============================================================================
// basic drawing and UI functions

void set_up_lighting(void)
{
   glEnable(GL_LIGHTING);
   {
      GLfloat ambient[4]={.3, .3, .3, 1};
      glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambient);
   }
   {
      GLfloat color[4]={.5, .5, .5, 1};
      GLfloat position[3]={0, 0, 0};
      glLightfv(GL_LIGHT0, GL_DIFFUSE, color);
      glLightfv(GL_LIGHT0, GL_SPECULAR, color);
      glLightfv(GL_LIGHT0, GL_POSITION, position);
      glEnable(GL_LIGHT0);
   }
   {
      GLfloat color[4]={.6, .6, .6, 1};
      GLfloat direction[3]={-1, -1, 0};
      glLightfv(GL_LIGHT1, GL_DIFFUSE, color);
      glLightfv(GL_LIGHT1, GL_SPECULAR, color);
      glLightfv(GL_LIGHT1, GL_SPOT_DIRECTION, direction);
      glEnable(GL_LIGHT1);
   }
   {
      GLfloat color[4]={.2, .2, .2, 1};
      GLfloat direction[3]={.5, -1, 0};
      glLightfv(GL_LIGHT2, GL_DIFFUSE, color);
      glLightfv(GL_LIGHT2, GL_SPECULAR, color);
      glLightfv(GL_LIGHT2, GL_SPOT_DIRECTION, direction);
      glEnable(GL_LIGHT2);
   }
}

void set_up_view(void)
{
   glViewport(0, 0, (GLsizei)winwidth, (GLsizei)winheight);

   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
   gluPerspective(fovy, winwidth/(float)winheight, near_clip, far_clip);

   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
   set_up_lighting(); // in camera-space: the lights "stick" to the camera
   // Here you should write transformations to define world-to-camera.
   // The camera should be located at camera_distance away from the world-space
   // origin and should point directly as it, but it should be rotated around
   // the world's y-axis and camera's x-axis according to the angles stored in
   // heading and pitch.
   // @@@@@@@@@@@@
}

void reshape(int w, int h)
{
   winwidth=w;
   winheight=h;
   glutPostRedisplay(); // better redraw in case the window changed size
}

void display(void)
{
   glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

   // establish the perspective projection and the world-to-camera model-view transform
   set_up_view();

   // draw your masterpiece!
   draw_marshmallow_man();

   // and make it visible on the screen
   glutSwapBuffers();
}

void mouse_click(int button, int state, int x, int y)
{
   if(state==GLUT_DOWN){
      old_mouse_x=x;
      old_mouse_y=y;
   }
}

void mouse_drag(int x, int y)
{
   int dx=x-old_mouse_x; // how far mouse was dragged horizontally
   int dy=y-old_mouse_y; // how far mouse was dragged vertically
   // here you should update heading and pitch according to dx and dy
   // @@@@@@@@@@@@@@@@
   // save the updated mouse position
   old_mouse_x=x;
   old_mouse_y=y;
   // tell GLUT we need to re-render for the altered camera
   glutPostRedisplay();
}

int main(int argc, char **argv)
{
   // set up GLUT
   glutInit(&argc, argv);
   glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
   glutInitWindowSize(winwidth, winheight);
   glutCreateWindow("cs314 assignment 2");
   glutReshapeFunc(reshape);
   glutDisplayFunc(display);
   glutMouseFunc(mouse_click);
   glutMotionFunc(mouse_drag);

   // set up OpenGL with the basics we need
   glEnable(GL_DEPTH_TEST);
   glClearColor(0, 0, 0, 0);
   glClearDepth(1);
   glEnable(GL_NORMALIZE);

   // and turn over control to GLUT and the callbacks
   glutMainLoop();
}

